
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>

#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include <errno.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <signal.h>

extern int errno;
#ifdef QDOS
#include "version.h"
#include <qdos.h>
char _prog_name[] = "mail";
char _version[] = MAIL_VERSION;
char _copyright[] = "(c) jrh 1998";
void (*_consetup)() = consetup_title;
#endif

#include "mailparse.h"

char *maildate (void)
{
    time_t t;
    t = time (NULL);
    return ctime (&t);
}


void sendtext (int s, char *line)
{
    if (write (s, line, strlen (line)) != strlen (line))
    {
	perror("send");
        exit(EXIT_FAILURE);
    }
}

int checkOK (int s, char *line, int resp, MAILDATA *m)
{
    char buf[256];
    int i;

    if ((i = write (s, line, strlen (line))) != strlen (line))
    {
	perror("send");
        exit(EXIT_FAILURE);
    }

    if(m->verbose == 2)
    {
        fputs("> ", stderr);
        fputs(line, stderr);
    }
    
    if ((i = read (s, buf, sizeof(buf)-1)) < 6)
    {
	perror("read");
        exit(EXIT_FAILURE);
    }
    if(m->verbose == 2)
    {
        buf[i] = 0;
        fputs("< ", stderr);
        fputs(buf, stderr);
    }


    if ((i = atoi (buf)) != resp)
    {
	if (m->verbose)
	{
	    fprintf (stderr,
                     "Server error %d, expected %d\nResponse: %sCommand : %s",
                     i, resp, buf, line);
	}
        exit(EXIT_FAILURE);
    }

    /* extended messages */
    while (*(buf+3) == '-')
    {
	if ((i = read (s, buf, sizeof(buf)-1)) < 6 )
        {
            perror("read");
            exit(EXIT_FAILURE);
        }
        if(m->verbose == 2)
        {
            buf[i] = 0;
            fputs("< ", stderr);
            fputs(buf, stderr);
        }
    }
    return 0;
}

void sendmail (int s, MAILDATA *m, char **to)
{
    FILE *ip = NULL,*sf;
    
    char **p, buf[BUFSIZ], fn[PATH_MAX];

    if(*to == NULL)
    {
        /* No recipients, let's hope there's a header then */        
        short off = 0, into = 0, incc = 0;
        to = malloc((sizeof(char*)*256));
        p = to;
        sprintf(fn,"%smailer.%x", m->tmp, getpid());
        
        ip = fopen(fn, "w+");        
        if(ip)
        {
            while(fgets(buf, sizeof(buf)-1,stdin))
            {
                fputs(buf, ip);

                if(into == 1 && *buf > ' ') into = 2;
                if(incc == 1 && *buf > ' ') incc = 2;

                if((into == 0) && strncasecmp(buf, "To:", 3) == 0)
                {
                    into = 1;
                    off = 3;
                }
                if((incc == 0) && strncasecmp(buf, "Cc:", 3) == 0)
                {
                    incc = 1;
                    off = 3;
                }
                
                if((into ==  1) || (incc == 1))
                {
                    char *p1,*p2;
                    
                    if((p1 = strchr(buf, '\n'))) *p1 = 0;
                    for(p1 = buf+off; (p2 = strtok(p1,", \t")) ; p1 = NULL)
                    {
                        if(*p2 > ' ')
                        {
                            *p = strdup(p2);
                            p++;
                        }
                    }
                    off = 0;
                }
            }
            *p = NULL;
            rewind (ip);
        }
        else
        {
            perror("tmp w+");
            exit(EXIT_FAILURE);
        }
    }
    else ip = stdin;
    
    
    sprintf (buf, "MAIL FROM:<%s@%s>\r\n", m->user, m->host);
    checkOK (s, buf, 250, m);

    for (p = to; *p; p++)
    {
        sprintf (buf, "RCPT TO:<%s>\r\n", *p);
        checkOK(s, buf, 250, m);
        if (m->verbose)
            fprintf (stderr, "Mail sent to %s\n", *p);
    }

    checkOK (s, "DATA\r\n", 354, m);
    if(ip == stdin)
    {
        sprintf (buf, "From: %s@%s (%s)\r\n", m->user, m->host, m->name);
        sendtext (s, buf);
        sendtext (s, "To: ");

        for (p = to; *p;)
        {
            sprintf (buf, "%s", *p);
            p++;
        
            if (*p != NULL)
                strcat (buf, ", ");
            sendtext (s, buf);
        }
        sendtext (s, "\r\n");    
    }

    sprintf (buf, "Date: %s", maildate());
    sendtext (s, buf);
    
    sendtext (s, "X-Mailer: qlmailer\r\n");
    

    while(fgets(buf, sizeof(buf)-1, ip))
    {
        sendtext(s, buf);
    }

    if(m->sigfile)
    {
        if((sf = fopen(m->sigfile,"r")))
        {
            sendtext (s, "\r\n");    
            while(fgets(buf, sizeof(buf)-1, sf))
            {
                sendtext(s, buf);
            }
            fclose(sf);
        }
    }
    checkOK (s, ".\r\n", 250, m);

    if(ip != stdin)
    {
        for (p = to; *p;p++)
        {
            free(*p);
        }
        fclose(ip);
        unlink(fn);
        free(to);
    }

}

int main (int ac, char **av)
{
    int s, i, c;
    char buf[128];
    MAILDATA *m;
    struct sockaddr_in addr;
    struct hostent *h;
    extern MAILDATA * MailParams(MAILDATA*, char *);
    char *fn = NULL;
    
    i = 0;
    
    while ((c = getopt (ac, av, "vf:to:")) != EOF)
    {
	switch (c)
	{
            case 'f':
                fn = optarg;
                break;
	    case 'v':
		i++;
		break;
            case 't':
            case 'o':
                    /* Just for (q)lynx */
            default:
                break;
        }
    }

    m = MailParams(NULL, fn);
    m->verbose = i;
    
    if(m->smtphost == NULL)
    {
        fputs("No SMTP host\n", stderr);
        exit(EXIT_FAILURE);
    }

    if(m->host == NULL)
    {
        if((m->host = malloc(256)) == NULL || gethostname(m->host, 256))
        {
            perror("Host name");
            exit(EXIT_FAILURE);
        }
    }

    if(m->user == NULL)
    {
        if((m->user = getenv("USER")) == NULL)
        {
            perror("UserName");
            exit(EXIT_FAILURE);
        }
    }
    
    
    if (NULL == (h = gethostbyname (m->smtphost)))
    {
	fprintf (stderr, "Unknown host %s\n", m->smtphost);
        exit (EXIT_FAILURE);
    }

    addr.sin_family = AF_INET;
    addr.sin_port = htons (25);
    addr.sin_addr = *((struct in_addr *) h->h_addr);

    if ((s = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
	perror ("socket");
        exit (EXIT_FAILURE);
    }

    if (connect (s, (struct sockaddr *) &addr, sizeof (addr)) == -1)
    {
	perror ("connect");
        exit (EXIT_FAILURE);
    }

    checkOK (s, "", 220, m);
    sprintf (buf, "HELO %s\r\n", (m->helo) ? m->helo : m->host);
    checkOK (s, buf, 250, m);
    sendmail (s, m, av + optind);
    checkOK (s, "QUIT\r\n", 221, m);
    shutdown (s, 2);
    close (s);
    return 0;
}
