#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>

#include "mailparse.h"

static char *strim (char *s)
{
    char *p;
    short n;

    n = strlen(s);
    p = s + n;
    while(*--p <= ' ' && n--)
    {
        *p = '\0';
    }
    return s;
}

static void  pString (char **p, char *s,...)
{
    if(*s)
    {
        *p = strdup(s);
    }
}

static void  pDir (char **p, char *s,...)
{
    int n;
    
    if(*s)
    {
        n = strlen(s);
        if(*(s+n-1) != SEPC)
        {
            *(s+n) = SEPC;
            *(s+n+1) = 0;
        }
        *p = strdup(s);
    }
}

#if 0
static void  pInt4 (long *p, char *s,...)
{
    *p  = atoi(s);
}

static void  pInt2 (short *p, char *s,...)
{
    *p  = (short) atoi(s);
}

static void pChar (short *p, char *s,...)
{
    *p = *s;
}

static void  pHex (char *p, char *s,...)
{
    va_list va;
    int n;
    
    if(*s)
    {
        va_start(va, s);
        n = va_arg(va, int);
	while(*s && --n)
	{
	    *(unsigned char *)p = strtoul(s, &s, 16);
	    p++;
	}
	*p = 0;
    }
}
#endif

# define VF (void (*)(void *,void *,...))
# define HUNKOFF(p1,p2) offsetof(p1,p2)

static PARSELIST pl[] = {
    {"TEMP",  VF pDir, HUNKOFF(MAILDATA, tmp)}, 
    {"HOST",  VF pString, HUNKOFF(MAILDATA, host)}, 
    {"SMTP",  VF pString, HUNKOFF(MAILDATA, smtphost)}, 
    {"USER",  VF pString, HUNKOFF(MAILDATA, user)}, 
    {"NAME",  VF pString, HUNKOFF(MAILDATA, name)},
    {"SIGF",  VF pString, HUNKOFF(MAILDATA, sigfile)},
    {"HELO",  VF pString, HUNKOFF(MAILDATA, helo)},        
    {NULL}
};


static MAILDATA * parsefile(char *pf, void *p, PARSELIST *plist)
{

    FILE *fp;
    
    if((fp = fopen(pf, "r")))
    {
	PARSELIST *ppl;
        char text [128];

	while(fgets(text, sizeof(text)-1, fp))
        {
            char *s;
            strim(text);
            for(ppl = plist; ppl->id; ppl++)
            {
                if(strncasecmp(text, ppl->id, strlen(ppl->id)) == 0)
                {
                    int l;
                    if((s = strchr(text, '=')))
                    {
                        s++;
                        l = strspn(s, " \t");
                    }
                    if(s)
                    {
                        (ppl->func)(((char *)p + ppl->offset), s + l,
                                    ppl->mval);
                        break;
                    }
                }
            }
        }
        fclose(fp);
    }
    else
    {
        p = NULL;
    }
    return p;
}

MAILDATA *MailParams (MAILDATA *p, char *pf)
{
    if(p == NULL)
    {
        p = malloc(sizeof(MAILDATA));
        memset(p, 0, sizeof(MAILDATA));
        p->tmp = TEMPDIR;
        p->name = LUSER;
    }
    
    if(pf == NULL)
    {
	if((pf = getenv("MAILRC")) == NULL)
	{
            exit(EXIT_FAILURE);
	}
    }

    return parsefile(pf, p, pl);
}

#if 0
int main(int ac, char **av)
{
    MAILDATA *m = NULL;

    if((m = MailParams (m, "mail.dat")) != NULL)
    {
        printf ("Temp        : %s\n", m->tmp);
        printf ("Host        : %s\n", m->host);
        printf ("SMTP        : %s\n", m->smtphost);
        printf ("User        : %s\n", m->user);
        printf ("Name        : %s\n", m->name);
    }
    
    return 0;
}
#endif
